package xyz.tbvns.ui;

import lombok.Getter;
import org.reflections.Reflections;
import xyz.tbvns.ui.Windows.ModelFileSelector;

import java.util.*;

public class UIManager {
    private static List<Element> registeredElements = new ArrayList<>();
    private static List<MenuBarTab> registeredMenuBarTabs = new ArrayList<>();
    @Getter
    private static final HashMap<Class, Boolean> activeUI = new HashMap<>();

    public static void loadUi() {
        try {
            Reflections reflections = new Reflections("xyz.tbvns.ui.Windows");
            Set<Class<? extends Element>> windows = reflections.getSubTypesOf(Element.class);
            for (Class<? extends Element> element : windows) {
                registeredElements.add(element.getDeclaredConstructor().newInstance());
                activeUI.put(element.asSubclass(Element.class), false);
            }

            reflections = new Reflections("xyz.tbvns.ui.Elements");
            Set<Class<? extends Element>> elements = reflections.getSubTypesOf(Element.class);
            for (Class<? extends Element> element : elements) {
                registeredElements.add(element.getDeclaredConstructor().newInstance());
                activeUI.put(element.asSubclass(Element.class), true);
            }

            reflections = new Reflections("xyz.tbvns.ui.MenuBarTabs");
            Set<Class<? extends MenuBarTab>> tabs = reflections.getSubTypesOf(MenuBarTab.class);
            for (Class<? extends MenuBarTab> element : tabs) {
                registeredMenuBarTabs.add(element.getDeclaredConstructor().newInstance());
            }
            registeredMenuBarTabs.sort(Comparator.comparingInt(MenuBarTab::index));

            activeUI.put(ModelFileSelector.class, true);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static void render() {
        for (Element element : registeredElements) {
            boolean isEnabled = activeUI.get(element.getClass());
            if (isEnabled) {
                element.render();
            }
        }
    }

    public static void renderMenuBarTabs() {
        for (MenuBarTab tab : registeredMenuBarTabs) {
            tab.render();
        }
    }
}
